package com.ekexiu.project.platform.demand;

import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.Column;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.method.From;
import org.jfw.apt.orm.annotation.dao.method.OrderBy;
import org.jfw.apt.orm.annotation.dao.method.SetSentence;
import org.jfw.apt.orm.annotation.dao.method.Where;
import org.jfw.apt.orm.annotation.dao.method.operator.Delete;
import org.jfw.apt.orm.annotation.dao.method.operator.DeleteWith;
import org.jfw.apt.orm.annotation.dao.method.operator.Insert;
import org.jfw.apt.orm.annotation.dao.method.operator.LimitSelect;
import org.jfw.apt.orm.annotation.dao.method.operator.PageSelect;
import org.jfw.apt.orm.annotation.dao.method.operator.QueryVal;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectList;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectOne;
import org.jfw.apt.orm.annotation.dao.method.operator.Update;
import org.jfw.apt.orm.annotation.dao.method.operator.UpdateWith;
import org.jfw.apt.orm.annotation.dao.param.GroupSqlColumn;
import org.jfw.apt.orm.annotation.dao.param.GtEq;
import org.jfw.apt.orm.annotation.dao.param.LessThan;
import org.jfw.apt.orm.annotation.dao.param.Set;
import org.jfw.apt.orm.core.defaultImpl.IntHandler;
import org.jfw.apt.orm.core.defaultImpl.StringHandler;
import org.jfw.util.PageQueryResult;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;

@DAO
public interface DemandDao {

	@Insert
	int insert(Connection con, Demand demand) throws SQLException;

	@Update
	@Where("STATE=1 AND ACTIVED='1'")
	int update(Connection con, Demand demand) throws SQLException;

	@UpdateWith
	@Where("STATE=1 AND ACTIVED='1'")
	@From(Demand.class)
	int update(Connection con, String id, String creator, @Set String invalidDay) throws SQLException;

	@Insert
	int insert(Connection con, DemandProfessor p) throws SQLException;

	@UpdateWith
	@From(DemandProfessor.class)
	@SetSentence("ACTIVED='1'")
	int confirmWork(Connection con, String id, String uid) throws SQLException;

	@UpdateWith
	@Where("STATE=1 AND ACTIVED='1'")
	@SetSentence("STATE=2")
	@From(Demand.class)
	int updateConfirm(Connection con, String id, @Set String overTime) throws SQLException;

	@UpdateWith
	@Where("STATE=1 AND ACTIVED='1'")
	@SetSentence("STATE=3")
	@From(Demand.class)
	int close(Connection con, String id, @Set String closeTime) throws SQLException;

	@Insert
	int insert(Connection con, DemandCloseReason dcr) throws SQLException;

	@UpdateWith
	@Where("STATE>1 AND ACTIVED='1'")
	@SetSentence("ACTIVED='0'")
	@From(Demand.class)
	int invalid(Connection con, String id) throws SQLException;

	@Insert
	int insert(Connection con, DemandSyncLog log) throws SQLException;

	@Delete
	int delete(Connection con, DemandSyncLog log) throws SQLException;

	@LimitSelect
	@OrderBy("ORDER BY TID ASC")
	List<DemandSyncLog> limitQuery(Connection con, int rows) throws SQLException;
	
	@Nullable
	@SelectOne
	DemandProfessor queryProfessor(Connection con,String id,String uid)throws SQLException;

	@SelectOne
	@Nullable
	Demand query(Connection con, String id) throws SQLException;

	@PageSelect
	@OrderBy("ORDER BY CREATE_TIME DESC")
	PageQueryResult<Demand> pqByCreateTime(Connection con, @Nullable @GroupSqlColumn(handlerClass = StringHandler.class,value = {"TITLE LIKE ?", "DESCP LIKE ?", "LINK_ORG LIKE ?"}, isAnd = false) String key, @Nullable Integer state, @Nullable Boolean actived, @Nullable @GtEq String invalidDay, int pageSize, int pageNo) throws SQLException;

	@PageSelect
	@OrderBy("ORDER BY OVER_TIME DESC")
	PageQueryResult<Demand> pqByOverTime(Connection con,@Nullable @GroupSqlColumn(handlerClass = StringHandler.class,value = {"TITLE LIKE ?", "DESCP LIKE ?", "LINK_ORG LIKE ?"}, isAnd = false) String key,@Nullable Integer state,@Nullable Boolean actived,@Nullable @GtEq String invalidDay, int pageSize, int pageNo) throws SQLException;

	@PageSelect
	@OrderBy("ORDER BY CLOSE_TIME DESC")
	PageQueryResult<Demand> pqByCloseTime(Connection con,@Nullable @GroupSqlColumn(handlerClass = StringHandler.class,value = {"TITLE LIKE ?", "DESCP LIKE ?", "LINK_ORG LIKE ?"}, isAnd = false) String key,@Nullable Integer state,@Nullable Boolean actived,@Nullable @GtEq String invalidDay, int pageSize, int pageNo) throws SQLException;

	@PageSelect
	@OrderBy("ORDER BY INVALID_DAY DESC")
	PageQueryResult<Demand> pqInvalidDay(Connection con,@Nullable @GroupSqlColumn(handlerClass = StringHandler.class,value = {"TITLE LIKE ?", "DESCP LIKE ?", "LINK_ORG LIKE ?"}, isAnd = false) String key, @LessThan String invalidDay, @Nullable Integer state, @Nullable Boolean actived, int pageSize, int pageNo) throws SQLException;

	@QueryVal
	@From(DemandProfessor.class)
	@DefaultValue("0")
	@Column(value="COUNT(ID)",handlerClass= IntHandler.class)
	int proCount(Connection con,String id) throws SQLException;

	@SelectList
	List<DemandCloseReason> queryReason(Connection con, String id) throws SQLException;

	@PageSelect
	@OrderBy("ORDER BY ASS_TIME")
	PageQueryResult<DemandProfessor> queryComments(Connection con,@Nullable Boolean actived, String id,int pageSize,int pageNo) throws SQLException;

	@DeleteWith
	@From(DemandCloseReason.class)
	int deleteReason(Connection con, String id) throws SQLException;

}
